' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
' // Jill of the Jungle
' // version 0.30
' // Started 02.13.05
' // Last revised 08.07.05
' // by Eikon (eikon@carolina.rr.com)
' // Support and graphics ripped by Caroline Lee (classical.roach@gmail.com)
' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//

' // Framework & Modules //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
Framework BRL.Max2D 
Import BRL.GLMax2D
Import BRL.D3D7Max2D
'Import BRL.PNGSaver
Import BRL.Basic
Import BRL.System
Import BRL.Audio
Import BRL.Retro
Import BRL.PNGLoader
Import BRL.OGGLoader
Import BRL.WAVLoader
Import BRL.FreeAudioAudio

Import "-lshell32"

' // Win32 API //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
Extern "win32"
	Function LoadIcon(hWnd, file:Byte Ptr, index) = "ExtractIconA@12"
	Function GetActiveWindow()
	Function GetDesktopWindow()
	Function GetWindowRect(hWnd, lpRect:Byte Ptr)
	Function SetWindowText(hWnd, lpString$z) = "SetWindowTextA@8"
	Function SetWindowPos(hWnd, after, x, y, w, h, flags)
	Function GetWindowLong(hwnd, nIndex) = "GetWindowLongA@8"
	Function SetWindowLong(hwnd, index, nIndex) = "SetWindowLongA@12"
	Function SetClassLong(hWnd, nIndex, dwNewLong) = "SetClassLongA@12"
	Function GetSystemMenu(hWnd, revert)
	Function GetMenuItemCount(hMenu)
	Function RemoveMenu(hMenu, position, flags)
	Function ShowCursor(bShow)
	Function GetTickCount()
End Extern

Type lpRECT
	Field l, t, r, b
End Type

' // Tile Constants //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
Const TILE_APPLE = 120, TILE_GEM = 121, TILE_GEMLOCK = 122
Const TILE_SPIKE = 123, TILE_MSPIKE = 124, TILE_TORCH = 125
Const TILE_START = 136, TILE_VSCROLL = 101
Const TILE_HSCROLL = 100, TILE_KEY = 130, TILE_LAVA = 135
Const TILE_LOCKB = 129, TILE_LOCKT = 128, TILE_FADER = 127
Const TILE_WATERM = 131, TILE_WATERT = 132, TILE_WATERR = 133
Const TILE_WATERL = 134

' // Create Window %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
Const GFX_WIDTH = 320, GFX_HEIGHT = 200

' // User.cfg //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
Global cfgSound, cfgMusic, screenMode, cfgShowFPS, cfgShowVMem, cfgVsync, cfgRenderer
Global MyMilliSecs = 0, LastMilliSecs = GetTickCount()

LoadCfg

If screenMode = 2 Then BIT_DEPTH = 0 Else BIT_DEPTH = 32
If cfgVsync = 1 Then HERTZ = 60 Else HERTZ = -1
If cfgRenderer = 1 Then SetGraphicsDriver GLMax2DDriver() Else SetGraphicsDriver D3D7Max2DDriver()
Graphics GFX_WIDTH, GFX_HEIGHT, BIT_DEPTH, HERTZ ' | SOFTSYNC
SeedRnd GetTickCount()

If screenMode = 2 Then
	Local hWnd% = GetActiveWindow()

	Local style = GetWindowLong(hWnd, -16)
	SetWindowLong hWnd, -16, style + $80000 + $20000

	Local hMenu = GetSystemMenu(hWnd, 0)
	Local iMenu = GetMenuItemCount(hMenu)
	RemoveMenu hMenu, iMenu - 1, $1000 + $400
	RemoveMenu hMenu, iMenu - 2, $1000 + $400

	SetWindowText hWnd, "Jill of the Jungle"
	Local icon = LoadIcon(hWnd, "icon.ico", 0)
	SetClassLong hWnd, -14, icon

	Local desk_hWnd% = GetDesktopWindow(), l:lpRect = New lpRECT
	GetWindowRect desk_hWnd, l:lpRECT 

	SetWindowPos hWnd, -2, (l.r / 2) - (GFX_WIDTH / 2), (l.b / 2) - (GFX_HEIGHT / 2), 0, 0, 1
	l:lpRECT = Null
EndIf

' // Types //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
Global o:Obj, objList:TList = New TList

Type Tile
	Field F%, X%, Y%
	Field Alp#, Rot%, Solid%
End Type

Type Obj
	Field X#, Y#, T%, F%, txt$, ID%, G#
	Field TX%, TY%, C[3], A%, R#
	Field Alp#, Rot%
End Type

Global f:objFire, fireList:TList = New TList
Type objFire
	Field OX, OY
	Field myFire[16, 38], myPal[256, 3]
	
	Function CreateFire:objFire(X, Y, C)
		Local f:objFire = New objFire, i, a, b
		f.OX = X; f.OY = Y
		For i = 0 To 85
			If C = 0 Then f.myPal[i, 0] = i * 3; f.myPal[i, 1] = 0; f.myPal[i, 2] = 0 ' Red
			If C = 1 Then f.myPal[i, 0] = 0; f.myPal[i, 1] = 0; f.myPal[i, 2] = i * 3 ' Blue
			If C = 2 Then f.myPal[i, 0] = 0; f.myPal[i, 1] = i * 3; f.myPal[i, 2] = 0 ' Green
		Next
		For i = 0 To 85
			If C = 0 Then f.myPal[i + 85, 0] = 255; f.myPal[i + 85, 1] = i * 3; f.myPal[i + 85, 2] = 0 ' Red
			If C = 1 Then f.myPal[i + 85, 0] = 0; f.myPal[i + 85, 1] = i * 3; f.myPal[i + 85, 2] = 255 ' Blue
			If C = 2 Then f.myPal[i + 85, 0] = 0; f.myPal[i + 85, 1] = 255; f.myPal[i + 85, 2] = i * 3 ' Green
		Next 
		For i = 0 To 85
			If C = 0 Then f.myPal[i + 170, 0] = 255; f.myPal[i + 170, 1] = 255; f.myPal[i + 170, 2] = i * 3 ' Red
			If C = 1 Then f.myPal[i + 170, 0] = i * 3; f.myPal[i + 170, 1] = 255; f.myPal[i + 170, 2] = 255 ' Blue
			If C = 2 Then f.myPal[i + 170, 0] = i * 3; f.myPal[i + 170, 1] = 255; f.myPal[i + 170, 2] = 255 ' Green
		Next 
		
		For b = 27 - 1 To 27 + 1
	     	For a = 8 - 3 To 8 + 3
				If a >= 0 And a <= 320 And b >= 0 And b <= 200 Then f.myFire[a, b] = 255
			Next
		Next

		Return f
	End Function
	
	Method RenderFire()
		If scrX >= OX - 15 And scrX - 1 <= OX And scrY >= OY - 10 And scrY - 1 <= OY Then ' On screen check
			tmpX = Int((OX + 5 - scrX) * 16 - scrollX) ' Convert to screen coords
			tmpY = Int((OY + 1 - scrY) * 16 - scrollY) 

			SetAlpha .8
			Local x, y, l, r, t, b, nc, nx#, ny, a, cooling = 8
			For y = 2 To 36 Step 2
	     		For x = 2 To 14 Step 2 
	               	l = myFire[x - 1,y]
		            r = myFire[x + 1,y]
	 	            t = myFire[x,y - 1]
		            b = myFire[x,y + 1]
		            nc = ((l + r + t + b) Shr 2) - (cooling + Rand(2)) 
		            If nc > 0
	                	nx# = x + Rnd(-.2,1.2)
	                    ny = y - Rand(2)
	                    myFire[nx, ny] = nc
						If myPal[nc, 0] < 78 And myPal[nc, 1] < 78 And myPal[nc, 2] < 78 Then
						Else
							SetColor myPal[nc, 0], myPal[nc, 1], myPal[nc, 2]
							DrawRect (x + tmpX) - 1, (y + tmpY) - 7, 2, 2
						EndIf
		            EndIf
        		Next
			Next

			For b = 27 - 1 To 27 + 1
		     	For a = 8 - 2 To 8 + 2
					If a >= 0 And a <= 320 And b >= 0 And b <= 200 Then myFire[a, b] = 255
				Next
			Next
			
			SetColor 255, 255, 255; SetAlpha 1
		EndIf

	End Method
	
End Type

' // Public //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
' // FPS Counter
Global fpsCount% = 0, fpsCurrent% = 0, fpsTime% = MilliSecs() + 1 

'; // Editor
Global numTiles% = 140, cLayer% = 1, vLayer% = 2

' // Player
Global pX# = 160, pY# = 80, pF% = 4, pTime% = MilliSecs(), pD% = 1, pJump = False, pRise = False
Global pG# = 2, pScore% = 0, idleTime% = MilliSecs(), bCol% = 0, pLife% = 160, pFall = False
Global pInv%[12]

' // Scrolling5
Global scrX% = 0, scrY% = 0, totalX, totalY#, maxX%, maxY%                            
Global scrollX# = 0, scrollY# = 0, backX# = 0, backY# = 0
Global bVScrollD = False, bVScrollU = False, bHScrollR = False, bHScrollL = False ' Scroll blockers

' // Status
Global statusTime% = MilliSecs(), statusTxt$, statusColor%

' // Animated Water
Global waterTime% = MilliSecs(), waterF%

' // Animated Lava
Global lavaTime = MilliSecs(), lavaF

' // Fader
Global faderTime = MilliSecs(), faderF

' // Arrays
Global highScore$[21]
Global Map:Tile[500, 500, 2]

Global Channel:TChannel

' // Local //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
Local bTitle = True, oldPY#

' // Images //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
AutoImageFlags MASKEDIMAGE

SetMaskColor 0, 0, 0
Global imgBack3:TImage = LoadImage("gfx/back3.png")
Global imgApple:TImage = LoadAnimImage("gfx/apple.png", 12, 11, 0, 4)
Global imgSky:TImage = LoadImage("gfx/sky.png")
Global imgSky2:TImage = LoadImage("gfx/sky2.png")
Global imgRadio:TImage = LoadAnimImage("gfx/radio.png", 7, 7, 0, 14)
Global imgBall:TImage = LoadAnimImage("gfx/ball.png", 6, 6, 0, 6)
Global imgGem:TImage = LoadAnimImage("gfx/gem.png", 12, 12, 0, 4)
Global imgCol:TImage = CreateImage(240, 160)
Global imgInventory:TImage = LoadAnimImage("gfx/inventory.png", 12, 10, 0, 2)
Global imgKey:TImage = LoadAnimImage("gfx/key.png", 16, 16, 0, 4)
Global imgLava:TImage = LoadAnimImage("gfx/lava.png", 16, 16, 0, 5)
Global imgFader:TImage = LoadAnimImage("gfx/fader.png", 16, 16, 0, 8)
Global imgWater:TImage = LoadAnimImage("gfx/water.png", 16, 16, 0, 12)

SetMaskColor 255, 0, 255
Global imgCursor:TImage = LoadImage("gfx/cur.png")
Global imgFont:TImage = LoadAnimImage("gfx/font.png", 5, 5, 0, 343)
Global imgTiles:TImage = LoadAnimImage("gfx/tiles.png", 16, 16, 0, numTiles)
Global imgJill:TImage = LoadAnimImage("gfx/jill.png", 16, 32, 0, 31)
Global imgParticles:TImage = LoadAnimImage("gfx/particles.png", 5, 5, 0, 12)
Global imgGemLock:TImage = LoadAnimImage("gfx/gemlock.png", 16, 16, 0, 4)
Global imgButtons:TImage = LoadAnimImage("gfx/buttons.png", 32, 8, 0, 6)
Global imgSpikes:TImage = LoadAnimImage("gfx/spikes.png", 16, 16, 0, 4)

' // Sounds //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
Global sfxTitle:TSound = LoadSound("sfx/title.ogg", 1)
Global sfxJump:TSound = LoadSound("sfx/jump.ogg")
Global sfxThrow:TSound = LoadSound("sfx/throw.ogg")
Global sfxApple:TSound = LoadSound("sfx/apple.ogg")
Global sfxFrog:TSound = LoadSound("sfx/frog.ogg")
Global sfxHurt:TSound = LoadSound("sfx/hurt.ogg")
Global sfxYea:TSound = LoadSound("sfx/yea.wav")
Global sfxLand:TSound = LoadSound("sfx/land.wav")
Global sfxSword:TSound = LoadSound("sfx/sword.ogg")
Global sfxWarp:TSound = LoadSound("sfx/warp.ogg")
Global sfxDeny:TSound = LoadSound("sfx/deny.ogg")
Global sfxGem:TSound = LoadSound("sfx/gem.ogg")

' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
' // Main
' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
#Main
SetBlend ALPHABLEND
HideMouse; ShowCursor False

LoadScores

If cfgMusic = True Then Channel = PlaySound(sfxTitle)
If bTitle = True Then TitleScreen ' Title screen

Repeat
Cls; SetColor 255, 255, 255

If cfgShowFPS = True Then
	fpsCount = fpsCount + 1
	If MilliSecs() >= fpsTime Then fpsCurrent = fpsCount; fpsCount = 0; fpsTime = fpsTime + 1000
EndIf

TileImage imgSky2, 0 - (backX * 2), 0 - (backY * 2)

'Print String(GetAlpha()) + ", " + String(GetRotation()) + ", " +  String(GetBlend())

'If pRise = True Then pF = 1 ' Rise

RenderTiles 0                                       ' Layer 0
DrawImage imgJill, Int(pX), Int(pY), pF             ' Jill
RenderTiles 1                                       ' Layer 1
RenderObj                                           ' Objects
For f:objFire = EachIn fireList; f.RenderFire; Next ' Fire
DrawImage imgBack3, 0, 0                            ' Frame
RenderInventory                                     ' Inventory
RenderHealth                                        ' Health
SetColor 0, 0, 0; DrawRect 0, 188, 320, 12          ' Status Area
RenderStatus                                        ' Status
SetColor 255, 255, 255

Rem
a = ((pX - 80) / 16) + scrX
b = ((pY - 16) / 16) + scrY; b:+3
Print a + ", " + b
If Map[a, b, 1] <> Null Then DrawRect Int((Map[a, b, 1].X + 5 - scrX) * 16 - scrollX), Int((Map[a, b, 1].Y + 1 - scrY) * 16 - scrollY), 16, 16
EndRem

tmpX% = 80; tmpY% = 16; tmp% = 0; tmp2% = 0; tmp4 = 0

If scrX <> 0 Then tmpX = 64; tmp = 1
If scrY <> 0 Then tmpY = 0; tmp2 = 1

bCol = -1; pFall = False ' Reset collision
For b = scrY - tmp2 To scrY + 10
	For a = scrX - tmp To scrX + 14
		If Map[a, b, 1] <> Null Then ' Layer 1
			SetColor 255, 0, 0
			'DrawRect pX - 32, pY - 32, 64, 64
			
			If Map[a, b, 1].F = TILE_FADER And faderF > 7 Then Goto StepOver1 ' Skip faded tiles
			
			If Map[a, b, 1].Solid = True And tmpX >= pX - 32 And tmpX <= pX + 32 And tmpY >= pY - 32 And tmpY <= pY + 64 Then
				'DrawRect Int(tmpX - scrollX), Int(tmpY - scrollY), 16, 16
				'tmp4 = 0
					
				If RectsOverlap(pX + 4, pY + 31, 8, 1, tmpX - scrollX, tmpY - scrollY, 16, 16) Then tmp4 = 1  ' Fall Check
				If RectsOverlap(pX + 17, pY + 3, 1, 26, tmpX - scrollX, tmpY - scrollY, 16, 16) Then bCol = 1 ' Right Side
				If RectsOverlap(pX - 1, pY + 3, 1, 26, tmpX - scrollX, tmpY - scrollY, 16, 16) Then bCol = 2  ' Left Side
				If RectsOverlap(pX + 4, pY - 2, 8, 1, tmpX - scrollX, tmpY - scrollY, 16, 16) Then            ' Head
					pJump = False; pG = 1
					bCol = 3
				EndIf

				If pY <= 16 Then bCol = 3 ' Top
				
				If tmp4 = 1 Then 'Or pFall = True Or bCol = 4 And bCol <> 1 And bCol <> 2 Then ' Riser
					tmp3 = 0
					If RectsOverlap(pX + 4, pY + 31, 8, 1, tmpX - scrollX, tmpY - scrollY, 16, 16) Then tmp3 = 0
					If RectsOverlap(pX + 4, pY + 30, 8, 1, tmpX - scrollX, tmpY - scrollY, 16, 16) Then tmp3 = 1
					If RectsOverlap(pX + 4, pY + 29, 8, 1, tmpX - scrollX, tmpY - scrollY, 16, 16) Then tmp3 = 2
					If tmp3 <> 0 Then 
						If pRise = False Then pY = pY - tmp3; pG = 2 Else pY:-.4

						'If bVScrollD = True Or bVScrollU = True Then scrollY = 0 ' FUCKED
						
						'If pJump = True Then 
						
						'If RectsOverlap(pX + 4, pY + 46, 8, 1, tmpX - scrollX, tmpY - scrollY, 16, 16) Then
							If cfgSound = True Then PlaySound sfxLand
						'EndIf
						pJump = False; pG = 2 ' Land
						'EndIf
					EndIf
				EndIf		
			EndIf	
		EndIf
		#StepOver1
		tmpX = tmpX + 16
	Next
	tmpY = tmpY + 16
	If scrX = 0 Then tmpX = 80 Else tmpX = 64
Next

'Print bCol + ", " + tmp4

' Horizontal no scroll check
bHScrollR = False; bHScrollL = False
For a = scrY To scrY + 10 ' Check for no scroll tile
	If Map[scrX + 14, a, 0] <> Null Then
		If Map[scrX + 14, a, 0].F = TILE_HSCROLL Then bHScrollR = True; Exit
	EndIf
	If Map[scrX, a, 0] <> Null Then
		If Map[scrX, a, 0].F = TILE_HSCROLL Then bHScrollL = True; Exit
	EndIf
	If Map[scrX + 14, a, 1] <> Null Then 
		If Map[scrX + 14, a, 1].F = TILE_HSCROLL Then bHScrollR = True; Exit
	EndIf
	If Map[scrX, a, 1] <> Null Then
		If Map[scrX, a, 1].F = TILE_HSCROLL Then bHScrollL = True; Exit
	EndIf
	'DrawRect (14 * 16) + 80, (a * 16) + 16, 16, 16
Next

If tmp4 = 0 Then ' Fall
	bVScrollD = False; bVScrollU = False
	For a = scrX To scrX + 15 ' Check for no scroll tile
		If Map[a, scrY + 9, 0] <> Null Then
			If Map[a, scrY + 9, 0].F = TILE_VSCROLL Then bVScrollD = True; Exit
		EndIf
		If Map[a, scrY, 0] <> Null Then
			If Map[a, scrY, 0].F = TILE_VSCROLL Then bVScrollU = True; Exit
		EndIf
		If Map[a, scrY + 9, 1] <> Null Then 
			If Map[a, scrY + 9, 1].F = TILE_VSCROLL Then bVScrollD = True; Exit
		EndIf
		If Map[a, scrY, 1] <> Null Then
			If Map[a, scrY, 1].F = TILE_VSCROLL Then bVScrollU = True; Exit
		EndIf
	Next

	If (pJump = False And bVScrollD = False) Or pY > 128 Then
		If pY >= 98 And scrY < maxY - 9 Then ' Scroll down
			totalY:+3
			scrollY:+3
			backY:+.5
		Else 
			If scrY >= maxY - 9 Then scrollY = 0 ' Align
			pY:+3
		EndIf
		pFall = True			
	Else
		If bVScrollD = True And pJump = False Then ' No Scroll Tile present
			If scrY >= maxY - 9 Then scrollY = 0   ' Align
			pY:+3
			pFall = True	
		EndIf
	EndIf
EndIf

'Print "Left: " + bHScrollL + " Right: " + bHScrollR + " Up: " + bVScrollU + " Down: " + bVScrollD

Rem
If tmp4 = 0 Then ' Fall
	If pJump = False Then 
		If pY >= 98 And scrY < maxY - 9 Then ' Scroll down
			totalY:+3
			scrollY:+3
			backY:+.5
		Else 
			If scrY >= maxY - 9 Then scrollY = 0 ' Align
			pY:+3
		EndIf
		bCol = 0			
	EndIf
EndIf
End Rem

' Horizontal no scroll check
Rem
tmpFail = False
For a = scrY To scrY + 11 ' Check for no scroll tile
	If Map[scrX + 15, a, 0] <> Null Then
		If Map[scrX + 15, a, 0].F = TILE_HSCROLL Then tmpFail = True; Exit
	EndIf
	If Map[scrX + 15, a, 1] <> Null Then 
		If Map[scrX + 15, a, 1].F = TILE_HSCROLL Then tmpFail = True; Exit
	EndIf
	'DrawRect (14 * 16) + 80, (a * 16) + 16, 16, 16
Next
If tmpFail = True Then bNoHorzScroll = True ' Block right
End Rem

bmpText 0, 66 - (Len(String(pScore)) - 1) * 6, 123, 1, pScore
If cfgShowFPS = True Then bmpText 0, 5, 190, 2, fpsCurrent

SyncAnimation ' Update global animation timers
			
' Collision rects
Rem
SetColor 255, 0, 0
DrawRect Int(pX + 4), Int(pY + 32), 8, 1
DrawRect Int(pX + 17), Int(pY + 3), 1, 26
DrawRect Int(pX - 1), Int(pY + 3), 1, 26
DrawRect Int(pX + 4), Int(pY - 2), 8, 1
SetColor 255, 255, 255
EndRem

'Color 255, 0, 0; Rect 80, 16, 232, 160, 0
'Color 255, 0, 0; Rect 80, 16, 232, 160, 0
'Color 255, 0, 0; Text 1, 1, scrX + ", " + scrY + " - " + scrollY
'Text 50, 1, maxY
	
'Delete Each Tile
'EndIf

'If cfgShowVMem = True Then bmpText 0, 290, 190, 80, 244, 244, (TotalVidMem() - AvailVidMem()) / 1024
'If cfgMusic = True And ChannelPlaying(Channel) = False Then Channel = PlaySound(sfxTitle)

If KeyHit(KEY_F1) Then SaveScreen                       ' Save screenshot
If KeyHit(KEY_F2) Then Print MemAlloced() / 1024 + "kbs / " + MemUsage() / 1024 + "kbs"
If KeyHit(KEY_ESCAPE) Then FlushKeys(); TitleScreen ' Exit

PlayerInput

FlushMem; Flip
Forever	

' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
' // RenderInventory
' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
Function RenderInventory()

Local tmpX% = 11, tmpY% = 133

For i = 0 To 11
	If pInv[i] <> 0 Then DrawImage imgInventory, tmpX, tmpY, pInv[i] - 1
	tmpY:+14
	If i = 2 Or i = 5 Or i = 8 Then tmpY = 133; tmpX:+16
Next

End Function


' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
' // SyncAnimation
' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
Function SyncAnimation()

If MilliSecs() >= waterTime + 200 Then ' Sync animated water
	If waterF < 2 Then waterF:+1 Else waterF = 0
	waterTime = MilliSecs()
EndIf

If MilliSecs() >= lavaTime + 200 Then  ' Sync animated lava
	If lavaF < 4 Then lavaF:+1 Else lavaF = 0
	lavaTime = MilliSecs()
EndIf

If faderF <> 3 Then                    ' Sync fading blocks
	If MilliSecs() >= faderTime + 350 Then
		If faderF < 13 Then faderF:+1 Else faderF = 0
		faderTime = MilliSecs()
	EndIf
Else
	If MilliSecs() >= faderTime + 2000 Then
		If faderF < 10 Then faderF:+1 Else faderF = 0
		faderTime = MilliSecs()
	EndIf
EndIf

End Function

' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
' // TitleScreen
' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
Function TitleScreen()

' // Images //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
SetMaskColor 0, 0, 0
Local imgTitle:TImage = CreateImage(320, 200)
Local imgRadio:TImage = LoadAnimImage("gfx/radio.png", 7, 7, 0, 2)

SetMaskColor 255, 0, 255
Local imgBack1:TImage = LoadImage("gfx/back1.png")

' // Locals //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
Local ballTime% = MilliSecs(), ballF% = 0, choice% = 0, bOptions = False, oC%

Goto RenderTitle

#TitleStart
Repeat
	Cls

	If cfgShowFPS = True Then
		fpsCount = fpsCount + 1
		If MilliSecs() >= fpsTime Then fpsCurrent = fpsCount; fpsCount = 0; fpsTime = fpsTime + 1000
	EndIf

	DrawImage imgTitle, 0, 0
	DrawImage imgBall, 88, 88 + (choice * 8), ballF
	If MilliSecs() >= ballTime + 100 Then ' Animate ball
		If ballF < 5 Then ballF:+1 Else ballF = 0
		ballTime = MilliSecs()
	EndIf

	If cfgShowFPS = True Then bmpText 0, 5, 190, 4, fpsCurrent
	'If cfgShowVMem = True Then bmpText 0, 290, 190, 4, (TotalVidMem() - AvailVidMem()) / 1024

	If bOptions = False ' Title screen

		If KeyHit(KEY_ESCAPE) Then SaveCfg; End ' Exit

		oC = choice
		If KeyHit(KEY_UP) Then choice:-1 ElseIf KeyHit(KEY_DOWN) Then choice:+1
		If oC <> choice And choice = 3 Then FlushKeys()

		If choice >= 6 Then choice = 0
		If choice <= -1 Then choice = 5

		If choice = 4 Then ' Noisemaker
			bmpText 0, 160 - ((Len("hit keys 1 - 9") * 6) / 2), 190, 2, "hit keys 1 - 9"
	
			If cfgSound = True Then
				If KeyHit(KEY_1) Or KeyHit(KEY_NUM1) Then PlaySound sfxJump
				If KeyHit(KEY_2) Or KeyHit(KEY_NUM2) Then PlaySound sfxApple
				If KeyHit(KEY_3) Or KeyHit(KEY_NUM3) Then PlaySound sfxFrog
				If KeyHit(KEY_4) Or KeyHit(KEY_NUM4) Then PlaySound sfxHurt
				If KeyHit(KEY_5) Or KeyHit(KEY_NUM5) Then PlaySound sfxLand
				If KeyHit(KEY_6) Or KeyHit(KEY_NUM6) Then PlaySound sfxThrow
				If KeyHit(KEY_7) Or KeyHit(KEY_NUM7) Then PlaySound sfxWarp
				If KeyHit(KEY_8) Or KeyHit(KEY_NUM8) Then PlaySound sfxGem
				If KeyHit(KEY_9) Or KeyHit(KEY_NUM9) Then PlaySound sfxDeny
			EndIf
		EndIf

		If KeyHit(KEY_ENTER) Or KeyHit(KEY_SPACE) Then ' Enter
			Select choice
				
				Case 0 ' Play
'				HideMouse
				tmpFile$ = RenderMaps$()
'				ShowPointer
'				
				If tmpFile$ <> "" Then
					LoadMap tmpFile$, 0
					'UpdateCol 0
					pF% = 4; pTime% = MilliSecs(); pLife% = 160
					pD% = 1; pJump = False; pG# = 2; pScore% = 0; bCol% = 0
					For i = 0 To 11
						pInv[i] = 0
					Next
					Goto StepOut 
				EndIf
				
				Case 1 ' Editor
				'StopChannel Channel
				'Delete Each Tile
				FlushKeys()
				Editor
				choice = 0
		
				Case 2 ' Online scores
'				bmpText 0, 160 - ((Len("connecting") * 6) / 2), 190, 240, 80, 80, "connecting": Flip 0
'				
'				If FTP_Connect("s93151849.onlinehome.us", "u35755648-ba", "blitzter", "") Then
'					DeleteFile "maps/list.txt"
'					FTP_Get "list.txt", 0, 0, 0 ; Download maplist
'					CopyFile "list.txt", "maps/list.txt": DeleteFile "list.txt"
'					
'					Cls
'					
'					File = ReadFile("maps/list.txt"): tmpY = 1
'					Repeat
'						Text 5, tmpY, ReadLine(File)
'						tmpY = tmpY + 12
'					Until Eof(File)
'					CloseFile File	
'					
'					Flip 0
'					FlushKeys(): WaitKey(): FlushKeys()
'				Else
'					Text 5, 16, "Failed!": Flip 0: Delay 1000	
				'EndIf
		
				Case 3 ' Options
				choice = 0; bOptions = True
				Goto RenderOptions

				Case 5 ' Exit
				SaveCfg
				End 
	
			End Select
		EndIf
	
	Else ' Options

		If cfgSound = True Then tmpF = 0 Else tmpF = 1
		DrawImage imgRadio, 98, 87, tmpF

		If cfgMusic = True Then tmpF = 0 Else tmpF = 1
		DrawImage imgRadio, 98, 95, tmpF

		If cfgVsync = True Then tmpF = 0 Else tmpF = 1
		DrawImage imgRadio, 98, 103, tmpF

		If screenMode = 1 Then tmpS$ = "full" Else tmpS$ = "window"
		bmpText 0, 109, 112, 3, tmpS$

		If cfgRenderer = 1 Then tmpS$ = "opengl" Else tmpS$ = "direct3d"
		bmpText 0, 109, 120, 3, tmpS$

		If KeyHit(KEY_ESCAPE) Then ' Return to title
			FlushKeys()
			choice = 0; bOptions = False
			Goto RenderTitle
		EndIf

		If KeyHit(KEY_UP) Then choice:-1 ElseIf KeyHit(KEY_DOWN) Then choice:+1

		If choice >= 6 Then choice = 0
		If choice <= -1 Then choice = 5

		If KeyHit(KEY_ENTER) Or KeyHit(KEY_SPACE)
			Select choice
				
				Case 0
				If cfgSound = True Then cfgSound = False Else cfgSound = True

				Case 1
				If cfgMusic = True Then cfgMusic = False Else cfgMusic = True

				Case 2
				If cfgVsync = True Then cfgVsync = False Else cfgVsync = True

				Case 3
				If screenMode <= 1 Then screenMode:+1 Else screenMode = 1

				Case 4
				If cfgRenderer = 1 Then cfgRenderer = 0 Else cfgRenderer = 1

				Case 5
				FlushKeys()
				choice = 0; bOptions = False
				Goto RenderTitle

			End Select
		EndIf

	EndIf

	'If cfgMusic = True And ChannelPlaying(Channel) = False Then Channel = PlaySound(sfxTitle)

	'DrawText fpsCurrent, 1, 187
	FlushMem; Flip
Forever

#RenderTitle ' Prerender text and scores
Cls
DrawImage imgBack1, 0, 0
bmpText 0, 85, 80, 1, "pick a choice:"
bmpText 0, 109, 88, 3, "play"
bmpText 0, 109, 96, 3, "editor"
bmpText 0, 109, 104, 2, "online levels"
bmpText 0, 109, 112, 4, "options"
bmpText 0, 109, 120, 4, "noisemaker"
bmpText 0, 109, 128, 6, "quit"
RenderScores 11, 30
GrabImage imgTitle, 0, 0

Goto TitleStart

#RenderOptions ' Prerender options
DrawImage imgBack1, 0, 0
bmpText 0, 85, 80, 1, "pick a choice:"
bmpText 0, 109, 88, 3, "sound"
bmpText 0, 109, 96, 3, "music"
bmpText 0, 109, 104, 3, "vsync"
bmpText 0, 109, 128, 6, "back"
RenderScores 11, 30
GrabImage imgTitle, 0, 0
		
Goto TitleStart

#StepOut
FlushKeys()

' Free Resources
imgTitle = Null
imgBack1 = Null
imgRadio = Null
FlushMem

End Function

' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
' // Editor
' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//	
Function Editor()

' // Locals //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
Local ballTime% = MilliSecs(), ballF% = 0, choice% = 0, tileStart = 0
Local tileC% = 0, tileRot% = 0, tileAlpha# = 10, tileSolid% = 1, bGrid = True
Local oldMapName$, tmpCheck, tileChangeTime = MilliSecs()

' Reset settings
vLayer = 3; cLayer = 1; scrX% = 0; scrY% = 0

For b = 0 To 499
	For a = 0 To 499
		Map[a, b, 0] = Null; Map[a, b, 1] = Null
	Next
Next

SetMaskColor 0, 0, 0; Cls
Local imgBack2:TImage = LoadImage("gfx/back2.png")
Local imgGrid:TImage = CreateImage(232, 160)

For b = 0 To 160 Step 16
	For a = 0 To 232 Step 16
		SetColor 255, 255, 255; DrawRect a, b, 16, 16
		SetColor 0, 0, 0; DrawRect a + 1, b + 1, 14, 14
	Next
Next
GrabImage imgGrid, 0, 0

Repeat
Cls; SetColor 255, 255, 255

If cfgShowFPS = True Then
	fpsCount = fpsCount + 1
	If MilliSecs() >= fpsTime Then fpsCurrent = fpsCount; fpsCount = 0; fpsTime = fpsTime + 1000
EndIf

If bGrid = True Then DrawImage imgGrid, 8, 16 ' Grid

' Render Map
SetImageHandle imgTiles, 8, 8
tmpX = 16; tmpY = 24
For b = scrY To scrY + 9
	For a = scrX To scrX + 14
		If vLayer = 0 Or vLayer = 3 Then ' Only if visible
			If Map[a, b, 0] <> Null Then 
				SetRotation Map[a, b, 0].Rot; SetAlpha Map[a, b, 0].Alp
				DrawImage imgTiles, tmpX, tmpY, Map[a, b, 0].F
			EndIf
		EndIf
		
		If vLayer > 0 Then ' Only If visible
			If Map[a, b, 1] <> Null Then
				tmpCheck = False
				If vLayer = 1 And Map[a, b, 1].Solid = True Then tmpCheck = True  ' Layer 1
				If vLayer = 2 And Map[a, b, 1].Solid = False Then tmpCheck = True ' Layer 2
				If tmpCheck = True Or vLayer = 3 Then
					SetRotation Map[a, b, 1].Rot; SetAlpha Map[a, b, 1].Alp
					DrawImage imgTiles, tmpX, tmpY, Map[a, b, 1].F
				EndIf
			EndIf
		EndIf

		tmpX:+16
	Next
	tmpY:+16; tmpX = 16
Next
SetImageHandle imgTiles, 0, 0; SetRotation 0; SetAlpha 1

DrawImage imgBack2, 0, 0

' Status Text	
'bmpText 0, 263, 5, 5, "tiles"
bmpText 0, 5, 190, 3, scrX + "." + scrY
bmpText 0, 231, 179, 1, cLayer
If tileAlpha = 10 Then tmp$ = "x" Else tmp$ = Left(String(tileAlpha), 1)
bmpText 0, 157, 6, 1, tmp$
If tileSolid = 0 Then tmpF% = 3 Else tmpF% = 2 
DrawImage imgRadio, 93, 6, tmpF
If bGrid = 0 Then tmpF% = 3 Else tmpF% = 2 
DrawImage imgRadio, 54, 6, tmpF

RenderStatus 

If vLayer = 3 Then 
	bmpText 0, 220, 6, 1, "all"
Else
	bmpText 0, 226, 6, 1, vLayer
EndIf

Rem
	DrawImage imgButtons, 7, 178, 1
	bmpText 0, 15, 179, 244, 244, 244, "new"
	DrawImage imgButtons, 41, 178, 4
	bmpText 0, 46, 179, 244, 244, 244, "load"
	DrawImage imgButtons, 75, 178, 5
	bmpText 0, 80, 179, 244, 244, 244, "save"
	bmpText 0, 112, 180, 0, 0, 0, "layer:"
	bmpText 0, 191, 8, 0, 0, 0, "view:"
EndRem

If cfgShowFPS = True Then bmpText 0, 300, 190, 2, fpsCurrent
'If cfgShowVMem = True Then bmpText 0, 290, 190, 80, 244, 244, (TotalVidMem() - AvailVidMem()) / 1024

' Tile Select area
tmpF = tileStart; bExit = False; SetImageHandle imgTiles, 8, 8
For b = 24 To 172 Step 16
	For a = 256 To 308 Step 16
		If tileC + tileStart = tmpF Then ' Only rotate and alpha the active tile
			SetColor 255, 255, 255; SetAlpha tileAlpha / 10; SetRotation tileRot
			DrawImage imgTiles, a, b, tmpF		
			
			'glPolygonMode GL_BACK, GL_LINE SetRotation 0 ' Selection rect
			SetColor 244, 244, 80; SetAlpha 1; SetRotation 0
			DrawLine a - 8, b - 8, a + 7, b - 8; DrawLine a + 7, b - 8, a + 7, b + 7
			DrawLine a + 7, b + 7, a - 8, b + 7; DrawLine a - 8, b + 7, a - 8, b - 8			
			'glPolygonMode GL_BACK, GL_FILL

			SetRotation 0; SetColor 255, 255, 255; SetAlpha 1			
		Else
			DrawImage imgTiles, a, b, tmpF
		EndIf
		
		If tmpF < numTiles - 1 Then tmpF:+1 Else bExit = True; Exit
	Next
	If bExit = True Then Exit
Next
SetImageHandle imgTiles, 0, 0

Rem
tmpMX = MouseX(); tmpMY = MouseY()
If KeyDown(KEY_J) Then MoveMouse tmpMX - 2, tmpMY ElseIf KeyDown(KEY_L) Then MoveMouse tmpMX + 2, tmpMY ' Cursor control
If KeyDown(KEY_I) Then MoveMouse tmpMX, tmpMY - 2 ElseIf KeyDown(KEY_K) Then MoveMouse tmpMX, tmpMY + 2
EndRem

DrawImage imgCursor, MouseX(), MouseY() ' Sword cursor

' //////////////////////////////////////
' // INPUT
' //////////////////////////////////////
If MouseHit(1) Then
	If RectsOverlap(226, 178, 15, 8, MouseX(), MouseY(), 1, 1) Then     ' Tile layer
		If cLayer = 0 Then cLayer = 1 Else cLayer = 0
	ElseIf RectsOverlap(217, 5, 24, 8, MouseX(), MouseY(), 1, 1) Then   ' View layer
		If vLayer < 3 Then vLayer:+1 Else vLayer = 0 
	ElseIf RectsOverlap(268, 178, 24, 8, MouseX(), MouseY(), 1, 1) Then ' Rotate
		If tileRot < 270 Then tileRot:+90 Else tileRot = 0
	ElseIf RectsOverlap(137, 5, 13, 8, MouseX(), MouseY(), 1, 1) Then   ' Alpha Down
		If tileAlpha > 1 Then tileAlpha:-1
	ElseIf RectsOverlap(169, 5, 13, 8, MouseX(), MouseY(), 1, 1) Then   ' Alpha Up
		If tileAlpha < 10 Then tileAlpha:+1
	ElseIf RectsOverlap(247, 178, 15, 8, MouseX(), MouseY(), 1, 1) Then ' Tiles Back
		If tileStart = 0 Then tileStart = 120 Else tileStart:-40
	ElseIf RectsOverlap(298, 178, 15, 8, MouseX(), MouseY(), 1, 1) Then ' Tiles Forward
		If tileStart < 120 Then tileStart:+40 Else tileStart = 0
	ElseIf RectsOverlap(93, 6, 6, 6, MouseX(), MouseY(), 1, 1) Then     ' Toggle Solid
		If tileSolid = 0 Then tileSolid = 1 Else tileSolid = 0
	ElseIf RectsOverlap(54, 6, 6, 6, MouseX(), MouseY(), 1, 1) Then     ' Toggle Grid
		If bGrid = 0 Then bGrid = 1 Else bGrid = 0
	EndIf
EndIf

'Print tileStart

If MouseDown(1) Or KeyDown(KEY_Z) And Right(statusTxt$, 6) <> "loaded" Then	
	If RectsOverlap(7, 178, 32, 8, MouseX(), MouseY(), 1, 1) Then ' New
		tmpPass = 0
		Repeat
			Cls
			bmpText 0, 2, 2, 2, "Confirm New Map"
			bmpText 0, 2, 10, 7, "Press Y or N"
			
			If KeyHit(KEY_Y) Then tmpPass = 1 ElseIf KeyHit(KEY_N) Then tmpPass = 2
			FlushMem; Flip
		Until tmpPass <> 0
		FlushKeys
		
		If tmpPass = 1 Then ' Yes
			For b = 0 To 499
				For a = 0 To 499
					Map[a, b, 0] = Null; Map[a, b, 1] = Null
				Next
			Next
			oldMapName$ = ""
		EndIf
		
	ElseIf RectsOverlap(41, 178, 32, 8, MouseX(), MouseY(), 1, 1) Then ' Load
		tmpFile$ = RenderMaps$()
	
		If tmpFile$ <> "" Then
			SetStatus 2, tmpFile$ + " loaded"
			LoadMap tmpFile$, 1
			oldMapName$ = tmpFile$
		EndIf
	
	ElseIf RectsOverlap(75, 178, 32, 8, MouseX(), MouseY(), 1, 1) Then ' Save
		FlushKeys(); tmpFile$ = ""; bHit = False
		If oldMapName$ <> "" Then 
			tmpExt = Instr(oldMapName$, ".")
			tmpFile$ = Left(oldMapName$, tmpExt - 1)
		EndIf

		Repeat
			Cls
			bmpText 0, 2, 2, 3, "enter map name:"
			bmpText 0, 2, 10, 1, tmpFile$
	
			tmpGK = GetChar()
			If tmpGK >= 65 And tmpGK <= 90 Then bHit = True  ' Upper Case
			If tmpGK >= 97 And tmpGK <= 122 Then bHit = True ' Lower Case
			If tmpGK >= 48 And tmpGK <= 57 Then bHit = True  ' Lower Case
			If tmpGK = 32 Then bHit = True
			If bHit = True Then
				DebugLog Chr$(tmpGK) + " - " + t3mpGK
				tmpFile$ = tmpFile$ + Chr$(tmpGK)
				bHit = False
			Else ' Backspace check
				If tmpGK = 8 And Len(tmpFile$) >= 1 Then tmpFile$ = Left$(tmpFile$, Len(tmpFile$) - 1)
			EndIf
			
			If KeyHit(KEY_ESCAPE) Then FlushKeys(); Goto StepOut
			
			FlushMem; Flip
		Until KeyHit(KEY_ENTER)
		
		FlushKeys()
		SaveMap tmpFile$
		SetStatus 2, tmpFile$ + ".joj" + " saved"
		oldMapName$ = tmpFile$ + ".joj"

	ElseIf MouseX() >= 8 And MouseX() <= 240 And MouseY() >= 17 And MouseY() <= 174 Then ' Map click
		tmpX = ((MouseX() - 8) + (scrX * 16)) / 16; tmpY = ((MouseY() - 16) + (scrY * 16)) / 16
		Map[tmpX, tmpY, cLayer] = New Tile
		Map[tmpX, tmpY, cLayer].F = tileC + tileStart; Map[tmpX, tmpY, cLayer].Rot = tileRot
		Map[tmpX, tmpY, cLayer].Alp = tileAlpha / 10; Map[tmpX, tmpY, cLayer].Solid = tileSolid
		
	ElseIf MouseX() >= 248 And MouseX() <= 310 And MouseY() >= 16 And MouseY() <= 175 Then ' Tile click
		tmpC = 0
		For b = 16 To 175 Step 16
			For a  = 248 To 310 Step 16
				If MouseX() >= a And MouseX() <= a + 16 And MouseY() >= b And MouseY() <= b + 16 Then tileC = tmpC
				tmpC = tmpC + 1
			Next
		Next
	EndIf
EndIf

If MouseDown(2) Or KeyDown(KEY_X) Then
	tmpX = ((MouseX() - 8) + (scrX * 16)) / 16; tmpY = ((MouseY() - 16) + (scrY * 16)) / 16
	Map[tmpX, tmpY, cLayer] = Null
EndIf

'oldMZ% = MouseZ()
'If MouseZ() >= oldMZ Then tileC:+1
'If MouseZ() <= oldMZ Then tileC:-1

'Select MouseZ()
'	Case -1: tileC = tileC - 1
'	Case 1: tileC = tileC + 1
	
'End Select
'
'If tileC <= -1 Then tileC = numTiles - 1 ElseIf tileC >= numTiles Then tileC = 0


If MilliSecs() >= tileChangeTime + 50 Then tmpCheck = True Else tmpCheck = False' Delayed tile switching

If tmpCheck = True Then tileChangeTime = MilliSecs()
If KeyDown(KEY_LEFT) Or KeyDown(KEY_A) And scrX > 0 Then 
	If tmpCheck = True Then scrX = scrX - 1
ElseIf KeyDown(KEY_RIGHT) Or KeyDown(KEY_D) And scrX < 900 Then 
	If tmpCheck = True Then scrX = scrX + 1
ElseIf KeyDown(KEY_UP) Or KeyDown(KEY_W) And scrY > 0 Then 
	If tmpCheck = True Then scrY = scrY - 1
ElseIf KeyDown(KEY_DOWN) Or KeyDown(KEY_S) And scrY < 900 Then 
	If tmpCheck = True Then scrY = scrY + 1
ElseIf KeyDown(KEY_SHIFT) Then
	If tileC > tileStart And tmpCheck = True Then tileC:-1 Else tileC = tileStart + 39
ElseIf KeyDown(KEY_CONTROL) Then 
	If tileC < tileStart + 39 And tmpCheck = True Then tileC:+1 Else tileC = tileStart
Else
	tileChangeTime = MilliSecs() - 100
EndIf

If KeyHit(KEY_F2) Then Print MemAlloced() / 1024 + "kbs / " + MemUsage() / 1024 + "kbs"
If KeyHit(KEY_F1) Then SaveScreen ' Save PNG Screenshot

'If cfgMusic = True And ChannelPlaying(Channel) = False Then Channel = PlaySound(sfxTitle)

#StepOut
FlushMem; Flip
Until KeyHit(KEY_ESCAPE)

imgGrid = Null; imgBack2 = Null
'ShowMouse

FlushMem; FlushKeys
End Function	

' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
' // CreateObj
' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
Function CreateObj(ID, x, y, txt$)

o:Obj = New Obj ' Create score floater
Select ID
	Case 2 ' Score Floater
	o.X = x; o.Y = y; o.txt$ = txt$; o.T = MilliSecs() - 100; o.ID = ID
	o.TX = totalX; o.TY = totalY; o.G = 1

	Case 3 ' Particles
	o.X = x; o.Y = y; o.ID = ID; o.T = MilliSecs()
	o.TX = totalX; o.TY = totalY; o.A = Rand(0, 359); o.R = 0; o.G = Rnd(.1, 1) ' Speed
	o.C[0] = Rand(0, 2)
	If o.C[0] = 0 Then o.F = 0 ElseIf o.C[0] = 1 Then o.F = 4 ElseIf o.C[0] = 2 Then o.F = 8
	
End Select

objList.AddLast o
End Function

' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
' // RenderObj
' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
Function RenderObj()

Local Del, tmpX%, tmpY% 

For o:Obj = EachIn objList
	Del = False; SetRotation o.Rot

	Select o.ID
		
		Case 1 ' Apple
		If scrX >= o.X - 15 And scrX - 1 <= o.X And scrY >= o.Y - 10 And scrY - 1 <= o.Y Then ' On screen check
			tmpX = Int((o.X + 5 - scrX) * 16 - scrollX) ' Convert to screen coords
			tmpY = Int((o.Y + 1 - scrY) * 16 - scrollY) 
							
			DrawImage imgApple, tmpX, tmpY, o.F
								
			If MilliSecs() >= o.T + 150 Then
				If o.F < 3 Then o.F = o.F + 1 Else o.F = 0
				o.T = MilliSecs()
			EndIf
					
			If ImagesCollide(imgJill, pX, pY, pF, imgApple, tmpX, tmpY, o.F) Then
				pScore:+12
				If pLife < 130 Then pLife:+30 Else pLife = 160
				If cfgSound = True Then PlaySound sfxApple
				objList.Remove o
				SetStatus 3, "apples grant Jill health"

				CreateObj 2, tmpX + 2, tmpY, "12" ' Create score floater
			EndIf
		EndIf

		Case 2 ' Score floater
		If MilliSecs() >= o.T + 100 Then ' Change color
			Repeat; tmp = Rand(0, 3); Until tmp <> o.F ' Don't repeat colors
			o.F = tmp
			If tmp = 0 Then o.C[0] = 2 ' Orange
			If tmp = 1 Then o.C[0] = 4 ' Cyan
			If tmp = 2 Then o.C[0] = 3 ' Green
			If tmp = 3 Then o.C[0] = 5 ' Blue
			o.T = MilliSecs()
		EndIf

		bmpText 1, o.X + (o.TX - totalX), o.Y + (o.TY - totalY), o.C[0], o.txt$
	
		o.Y = o.Y - o.G; o.X = o.X + 1; o.G = o.G + .075
		If o.Y + (o.TY - totalY) <= 0 Then Del = 1

		Case 3 ' Particles
		If MilliSecs() >= o.T + 350 Then
			o.F = o.F + 1
			o.T = MilliSecs()
			If o.C[0] = 0 And o.F >= 4 Then Del = 1
			If o.C[0] = 1 And o.F >= 8 Then Del = 1
			If o.C[0] = 2 And o.F >= 12 Then Del = 1		
		EndIf
		
		If Del = 0 Then
			DrawImage imgParticles, o.R * Cos(o.A) + o.X + (o.TX - totalX), o.R * Sin(o.A) + o.Y + (o.TY - totalY), o.F
			o.R:+o.G ' Speed
		EndIf

		Case 4 ' Gem
		If scrX >= o.X - 15 And scrX - 1 <= o.X And scrY >= o.Y - 10 And scrY - 1 <= o.Y Then ' On screen check
			tmpX = Int((o.X + 5 - scrX) * 16 - scrollX) ' Convert to screen coords
			tmpY = Int((o.Y + 1 - scrY) * 16 - scrollY) 

			DrawImage imgGem, tmpX, tmpY, o.F
									
			If MilliSecs() >= o.T + 100 Then
				If o.F < 3 Then o.F = o.F + 1 Else o.F = 0
				o.T = MilliSecs()
			EndIf
			
			If ImagesCollide(imgJill, pX, pY, pF, imgGem, tmpX, tmpY, o.F) Then
				pScore:+23
				objList.Remove o
				If cfgSound = True Then PlaySound sfxGem

				CreateObj 2, tmpX + 2, tmpY, "23" ' Create score floater
				For i = 0 To 15 ' Create particles
					CreateObj 3, tmpX + 4, tmpY + 4, ""
				Next

				For i = 0 To 11 ' Add gem to inventory
					If pInv[i] = 0 Then pInv[i] = 1; Exit
				Next
			EndIf
		EndIf

		Case 5 ' Spikes
		If scrX >= o.X - 15 And scrX - 1 <= o.X And scrY >= o.Y - 10 And scrY - 1 <= o.Y Then ' On screen check
			tmpX = Int((o.X + 5 - scrX) * 16 - scrollX) ' Convert to screen coords
			tmpY = Int((o.Y + 1 - scrY) * 16 - scrollY) 

			If o.Rot <> 0 Then 
				SetImageHandle imgTiles, 8, 8
				DrawImage imgTiles, tmpX + 8, tmpY + 8, TILE_SPIKE
				SetImageHandle imgTiles, 0, 0
			Else
				DrawImage imgTiles, tmpX, tmpY, TILE_SPIKE
			EndIf
						
			'Color 255, 0, 0: Rect pX, pY, 16, 32, 0
			SetRotation 0
			If RectsOverlap(tmpX, tmpY, 16, 16, pX, pY, 16, 33) Then 
				pLife = pLife - 1
			EndIf
		EndIf
		
		Case 6 ' Gem lock
		If scrX >= o.X - 15 And scrX - 1 <= o.X And scrY >= o.Y - 10 And scrY - 1 <= o.Y Then ' On screen check
			tmpX = Int((o.X + 5 - scrX) * 16 - scrollX) ' Convert to screen coords
			tmpY = Int((o.Y + 1 - scrY) * 16 - scrollY) 

			DrawImage imgGemLock, tmpX, tmpY, o.F
					
			If MilliSecs() >= o.T + 250 Then
				If o.F < 3 Then o.F = o.F + 1 Else o.F = 0
				o.T = MilliSecs()
			EndIf
			
			If RectsOverlap(tmpX, tmpY, 16, 16, pX, pY + 1, 18, 32) Then 
				tmp = -1
				For i = 0 To 11
					If pInv[i] = 1 And i > tmp Then tmp = i
				Next
				If tmp <> - 1 Then ' Play has gem
					pInv[tmp] = 0
				
					' Remove it from collision
					Map[o.X, o.Y, 1] = Null
					objList.Remove o
	
					If cfgSound = True Then PlaySound sfxWarp
				EndIf
			EndIf
		EndIf

		Case 7 ' Moving Spikes
		If scrX >= o.X - 15 And scrX - 1 <= o.X And scrY >= o.Y - 10 And scrY - 1 <= o.Y Then ' On screen check
			tmpX = Int((o.X + 5 - scrX) * 16 - scrollX) ' Convert to screen coords
			tmpY = Int((o.Y + 1 - scrY) * 16 - scrollY) 

			If o.Rot <> 0 Then 
				SetImageHandle imgSpikes, 8, 8
				DrawImage imgSpikes, tmpX + 8, tmpY + 8, o.F
				SetImageHandle imgSpikes, 0, 0
			Else
				DrawImage imgSpikes, tmpX, tmpY, o.F
			EndIf
					
			If MilliSecs() >= o.T + 250 Then
				If o.F < 3 Then o.F = o.F + 1 Else o.F = 0
				o.T = MilliSecs()
			EndIf
					
			'Color 255, 0, 0: Rect pX, pY, 16, 32, 0
			SetRotation 0
			If RectsOverlap(tmpX, tmpY, 16, 16, pX, pY, 16, 33) Then 
				pLife = pLife - 1
			EndIf
		EndIf

		Case 8 ' Key
		If scrX >= o.X - 15 And scrX - 1 <= o.X And scrY >= o.Y - 10 And scrY - 1 <= o.Y Then ' On screen check
			tmpX = Int((o.X + 5 - scrX) * 16 - scrollX) ' Convert to screen coords
			tmpY = Int((o.Y + 1 - scrY) * 16 - scrollY) 

			DrawImage imgKey, tmpX, tmpY, o.F
									
			If MilliSecs() >= o.T + 150 Then
				If o.F < 3 Then o.F = o.F + 1 Else o.F = 0
				o.T = MilliSecs()
			EndIf
			
			If ImagesCollide(imgJill, pX, pY, pF, imgKey, tmpX, tmpY, o.F) Then
				objList.Remove o
				If cfgSound = True Then PlaySound sfxYea

				For i = 0 To 11 ' Add key to inventory
					If pInv[i] = 0 Then pInv[i] = 2; Exit
				Next
			EndIf
		EndIf
		
		Case 9 ' Key Door
		If scrX >= o.X - 15 And scrX - 1 <= o.X And scrY >= o.Y - 10 And scrY - 1 <= o.Y Then ' On screen check
			tmpX = Int((o.X + 5 - scrX) * 16 - scrollX) ' Convert to screen coords
			tmpY = Int((o.Y + 1 - scrY) * 16 - scrollY) 

			If o.F = 0 Then 
				DrawImage imgTiles, tmpX, tmpY + o.C[1], TILE_LOCKB
			Else
				DrawImage imgTiles, tmpX, tmpY - o.C[1], TILE_LOCKT
			EndIf
						
			If o.C[0] = 1 Then ' Open
				' Render adjacent tiles on top of the door
				If o.F = 0 And Map[o.X, o.Y + 1, 1] <> Null Then DrawImage imgTiles, tmpX, tmpY + 16, Map[o.X, o.Y + 1, 1].F
				If o.F = 1 And Map[o.X, o.Y - 1, 1] <> Null Then DrawImage imgTiles, tmpX, tmpY - 16, Map[o.X, o.Y - 1, 1].F
				
				If MilliSecs() >= o.T + 50 Then
					o.T = MilliSecs()		
					o.C[1]:+1						
				EndIf
			EndIf
			
			If RectsOverlap(tmpX, tmpY, 16, 16, pX, pY + 1, 18, 32) And o.C[0] = 0 Then 
				tmp = -1
				For i = 0 To 11
					If pInv[i] = 2 And i > tmp Then tmp = i
				Next
				If tmp <> -1 Then   ' Play has a key
					If o.F = 0 Then ' Only fire once
						pInv[tmp] = 0 
						If cfgSound = True Then PlaySound sfxWarp
					EndIf
					o.C[0] = 1 ' Open
					Map[o.X, o.Y, 1] = Null ' Remove it from collision
				EndIf
			EndIf
			
			If o.C[0] = 1 And o.C[1] >= 16 Then 		
				objList.Remove o ' Delete
			EndIf			
		EndIf
		
		Case 10 ' Fader
		If scrX >= o.X - 15 And scrX - 1 <= o.X And scrY >= o.Y - 10 And scrY - 1 <= o.Y Then ' On screen check
			tmpX = Int((o.X + 5 - scrX) * 16 - scrollX) ' Convert to screen coords
			tmpY = Int((o.Y + 1 - scrY) * 16 - scrollY) 

			If faderF <= 7 Then DrawImage imgFader, tmpX, tmpY, faderF
		EndIf
		
	End Select
		
	If Del = True Then objList.Remove o
Next

SetRotation 0
End Function

' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
' // PlayerInput
' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
Function PlayerInput()

If KeyDown(KEY_LEFT) And (Not KeyDown(KEY_RIGHT)) And bCol <> 2 Then ' Move Left
	If pFall = True Then ' Fall
		pF = 12
	Else             ' Run
		If pF < 23 Or pF > 30 Or pD = 1 Then pF = 23     ' Reset frame

		If MilliSecs() >= pTime + 80 Then                ' Running animation
			If pF < 30 Then pF = pF + 1 Else pF = 23
			pTime = MilliSecs()
		EndIf
	EndIf

	pD = 0 
	
	If bHScrollL = True Then
		pX = pX - 2 
	Else
		If scrX <= 0 Or pX >= 188 And scrX >= mapX - 14 Then ' Move Jill
			pX = pX - 2 
		Else                                                 ' Scroll screen
			scrollX = scrollX - 2; backX = backX - .5; totalX = totalX - 2
		EndIf
	EndIf
	
	If scrollX <= -16 Then scrollX = scrollX + 16; scrX = scrX - 1 ' Scroll Left

ElseIf KeyDown(KEY_RIGHT) And (Not KeyDown(KEY_LEFT)) And bCol <> 1 Then ' Move Right
	If pFall = True Then ' Fall
		pF = 10
	Else
		If pF < 14 Or pF > 21 Or pD = 0 Then pF = 14 ' Reset frame
	
		If MilliSecs() >= pTime + 80 Then            ' Running Animation
			If pF < 21 Then pF = pF + 1 Else pF = 14
			pTime = MilliSecs()
		EndIf
	EndIf

	pD = 1

	If bHScrollR = True Then ' Right scroll blocked
		pX = pX + 2
	Else                     ' Normal scrolling
		If pX >= 188 And scrX < maxX  - 14 Then ' Scroll screen
			scrollX = scrollX + 2; backX = backX + .5; totalX = totalX + 2
		Else                                    ' Move Jill
			pX = pX + 2
		EndIf
	EndIf
	
	If scrollX >= 16 Then scrollX = scrollX - 16; scrX = scrX + 1 ' Scroll Right

ElseIf KeyDown(KEY_DOWN) Then ' Duck
	pF = 8

ElseIf KeyDown(KEY_UP) Then   ' Look up
	pF = 1
Else                     ' Idle
	If pJump <> True Then 
		If pFall = True Then ' Fall
			pF = 7
		Else             ' Stand
			If pD = 0 Then pF = 22 Else pF = 13
		EndIf
	EndIf
EndIf

If KeyHit(KEY_Z) And pJump = False And pFall = False And bCol <> 0 Then ' Jump
	If cfgSound = True Then PlaySound(sfxJump)
	pG = 4; pJump = True
EndIf

If pJump = True And pG <= -3 Then pJump = False; pG = 3 ' Turn jump into a fall

If pJump = True Then ' Jump
	If pD = 0 Then pF = 11 Else pF = 9

	If pG > 0 Then ' Up
		If pY <= 54 And bVScrollU = False Then ' Scroll Up
			If scrY = 0 Then
				Select True
					Case scrollY <= -15 And scrollY >= -16
					scrY = 0; scrollY = 0 ' Align

					Case scrollY <= 0 And scrollY >= -3
					scrY = 0; scrollY = 0 ' Align

				End Select
				pY = pY - pG
			Else
				totalY = totalY  - pG
				scrollY = scrollY - pG
				backY = backY - (pG / 4)
				pY = (pY - pG) + pG
			EndIf
		Else
			pY = pY - pG
    	EndIf
		
	Else ' Down
		If pY >= 98 And scrY < maxY - 9 And bVScrollD = False Then ' Scroll Down
			totalY = totalY + 3
			scrollY = scrollY + 3
			backY = backY + .5
			pF = 7
			pY = (pY - pG) - 3
		Else
			If scrY >= maxY - 9 Then
				Select True
					Case scrollY >= 15 And scrollY <= 16 
					scrY = maxY - 9; scrollY = 0 ' Align
		
					Case scrollY >= 0 And scrollY <= 2
					scrY = maxY - 9; scrollY = 0 ' Align

				End Select
			EndIf
			pY = pY - pG
		EndIf
	EndIf
	If pG > -3 Then pG = pG - .15 Else pG = -3
EndIf

If scrollY >= 16 Then scrY = scrY + 1; scrollY = scrollY - 16 ' Scroll Down
If scrollY <= -16 Then 
	If scrY > 0 Then 
		scrY = scrY - 1; scrollY = scrollY + 16 ' Scroll Up
	Else
		';scrY = 0: scrollY = 0
	EndIf
EndIf

If bCol = 3 Then pJump = False ' Hit head - fall
End Function

' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
' // bmpText
' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
Function bmpText(i%, x%, y%, c%, txt$)
SetAlpha 1; SetColor 255, 255, 255

Local a%, tmp%

For a = 1 To Len(txt$)
	tmp = Asc(Lower$(Mid(txt$, a, 1)))

	Select True
		Case tmp = 32 ' Space
				
		Case tmp = 58 ' Colon
		DrawImage imgFont, x + (a - 1) * 6, y, 37 + (c - 1) * 49
		
		Case tmp = 45 ' Hyphen
		DrawImage imgFont, x + (a - 1) * 6, y, 36 + (c - 1) * 49
		
		Case tmp = 46 ' Period
		DrawImage imgFont, x + (a - 1) * 6, y, 38 + (c - 1) * 49
	
		Case tmp >= 48 And tmp <= 57 And i = 0 ' Big numbers
		DrawImage imgFont, x + (a - 1) * 6, y, tmp - 22 + (c - 1) * 49

		Case tmp >= 48 And tmp <= 57 And i = 1 ' Small numbers
		DrawImage imgFont, x + (a - 1) * 4, y, tmp - 9 + (c - 1) * 49
		
		Default ' A-Z
		DrawImage imgFont, x + (a - 1) * 6, y, tmp - 97 + (c - 1) * 49
		
	End Select
Next

SetColor 255, 255, 255
End Function

' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
' // LoadScores
' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
Function LoadScores()
	
Local i%, name$, score$

File = ReadFile("scores.txt")
i = 1
Repeat
	name$ = ReadLine(File)
	score$ = ReadLine(File)
	highScore$[i] = name$ + " " + score$
	' Print name$ + " " + score$
	i = i + 1
Until Eof(File)

CloseFile File
End Function

' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
' // RenderScores
' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
Function RenderScores(x, y)

Local i%, a%, tmp%, name$, score$, off%

For i = 1 To 20
	tmp = Instr(highScore$[i], " ", 1)
	If tmp > 0 Then
		name$ = Left$(highScore$[i], tmp - 1)
		score$ = Right$(highScore$[i], Len(highScore$[i]) - tmp)

		bmpText 0, x, y + (i - 1) * 7, 3, name$

		For a = 1 To Len(score$)
			tmp = Asc(Lower(Mid(score$, a, 1)))
			Select True
				Case tmp = 32 ' Space
		
				Case tmp >= 48 And tmp <= 57 ' Small numbers
				DrawImage imgFont, 65 - Len(score$) * 4 + (a * 4), y + (i - 1) * 7, tmp - 9 + 294
		
			End Select
		Next
	
	EndIf
Next

End Function

' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
' // LoadCfg
' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
Function LoadCfg()

Local File% = ReadFile("user.cfg")

ReadLine File; 
ReadLine File; cfgSound = Byte(ReadLine(File))    ' Sound
ReadLine File; cfgMusic = Byte(ReadLine(File))    ' Music
ReadLine File; screenMode = Byte(ReadLine(File))  ' Screen Mode
ReadLine File; cfgShowFPS = Byte(ReadLine(File))  ' Show FPS
ReadLine File; cfgShowVMem = Byte(ReadLine(File)) ' Show Mem
ReadLine File; cfgVsync = Byte(ReadLine(File))    ' VSYNC
ReadLine File; cfgRenderer = Byte(ReadLine(File)) ' Renderer

CloseFile File
End Function

' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
' // SaveCfg
' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
Function SaveCfg()

Local File% = WriteFile("user.cfg")

WriteLine File, "# Jill Engine generated user config file | " + CurrentDate$() + " | " + CurrentTime$()
WriteLine File, "# Digital Sound Effects"
WriteLine File, cfgSound	
WriteLine File, "# Digitized Music"
WriteLine File, cfgMusic
WriteLine File, "# Screen Mode (1 = Full | 2 = Window)"
WriteLine File, screenMode
WriteLine File, "# Display Frames Per Second"
WriteLine File, cfgShowFPS
WriteLine File, "# Display Used Video Memory"
WriteLine File, cfgShowVMem
WriteLine File, "# Enable Vertical Sync (limits framerate)"
WriteLine File, cfgVsync
WriteLine File, "# Renderer (1 = OpenGL | 2 = Direct3D)"
WriteLine File, cfgRenderer

CloseFile File
End Function

' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
' // RenderMaps
' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
Function RenderMaps$()
FlushKeys(); FlushMouse()

Local tmpY%, bNone, tmpFile$, file$, tmpC%, tmpN%, tmpN2% = 1

Repeat
	Cls
	Dir = ReadDir("maps/"); tmpY  = 2; tmpFile$ = ""; bNone = True
	
	tmpN% = 1
	Repeat 
		file$ = NextFile(Dir)

		If FileType("maps/" + file$) = 1 And Right(file$, 3) = "joj" Then
			If MouseX() >= 5 And MouseX() <= 5 + Len(file$) * 6 And MouseY() >= tmpY And MouseY() <= tmpY + 5 Then 
				tmpC = 3
				tmpFile$ = file$; tmpN2 = tmpN
			ElseIf tmpN = tmpN2 Then ' Keyboard
				tmpC = 3
				tmpFile$ = file$
			Else
				tmpC = 1
			EndIf
					
			bmpText 0, 5, tmpY, tmpC, file$
			tmpY:+7; tmpN:+1; bNone = False
		EndIf
	Until file$ = "" 
	CloseDir Dir
	
	bmpText 0, 5, 190, 7, "Use the cursor keys or the mouse to select a map"
	If bNone = True Then bmpText 0, 5, 2, 1, "no maps found..."

	DrawImage imgCursor, MouseX(), MouseY()

	If KeyHit(KEY_UP) Then 
		If tmpN2 > 1 Then tmpN2:-1 Else tmpN2 = tmpN - 1
	ElseIf KeyHit(KEY_DOWN) Then
		If tmpN2 = tmpN - 1 Then tmpN2 = 1 Else tmpN2:+1
	EndIf
	
	If KeyHit(KEY_ESCAPE) Then FlushKeys(); FlushMouse(); Return ""

	FlushMem; Flip
Until MouseHit(1) Or KeyHit(KEY_ENTER) And tmpFile$ <> ""

Return tmpFile$
End Function

' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
' // SetStatus
' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
Function SetStatus(c, txt$)

statusColor = c
statusTxt$ = txt$
statusTime% = MilliSecs()

End Function

' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
' // RenderStatus()
' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
Function RenderStatus()

Local tmpC

If MilliSecs() <= statusTime + 2000 Then

	If statusColor = 1 Then tmpC = 1 ' White
	If statusColor = 2 Then tmpC = 2 ' Red
	If statusColor = 3 Then tmpC = 3 ' Green
	If statusColor = 4 Then tmpC = 4 ' Cyan
	If statusColor = 5 Then tmpC = 5 ' Blue
	
	SetColor 255, 255, 255
	bmpText 0, 160 - ((Len(statusTxt$) * 6) / 2), 190, tmpC, statusTxt$
Else
	statusTxt$ = ""
EndIf

End Function

' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
' // LoadMap
' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
Function LoadMap(myfile$, i)

Local a, b, file, tmpX, tmpY, tmpF, tmpL, tmpR, tmpA#, tmpS, tmpMX, tmpMY, tmpVar
scrX = 0; scrY = 0

For b = 0 To 499 ' Clear map
	For a = 0 To 499
		Map[a, b, 0] = Null; Map[a, b, 1] = Null
	Next
Next

For o:Obj = EachIn objList; objList.Remove o; Next       ' Clear Types
For f:objFire = EachIn fireList; fireList.Remove f; Next ' Clear Fire

file = ReadFile("maps/" + myfile$)

Repeat
	tmpX  = ReadShort(file) ' X
	tmpY  = ReadShort(file) ' Y
	tmpF  = ReadByte(file)  ' Frame
	tmpL  = ReadByte(file)  ' Layer
	tmpR  = ReadShort(file) ' Rotation
	tmpA# = ReadFloat(file) ' Alpha
	tmpS  = ReadByte(file)  ' Solid
	
	If i = 0 Then ' Play Mode
	
		Select tmpF
			Case TILE_APPLE, TILE_SPIKE, TILE_GEM, TILE_GEMLOCK, TILE_MSPIKE, TILE_KEY, TILE_LOCKB, TILE_LOCKT, TILE_FADER ' Object Tiles
			o:Obj = New Obj		
			o.X = tmpX        ' X
			o.Y = tmpY        ' Y
			o.T = MilliSecs() ' Time
			o.Rot = tmpR      ' Rotation
			o.Alp = tmpA      ' Alpha
			
			If tmpF = TILE_APPLE Then       ' Apple
				o.ID = 1; o.F = Rand(0, 3)
			ElseIf tmpF = TILE_GEM Then     ' Gem				
				o.ID = 4; o.F = Rand(0, 3)  
			ElseIf tmpF = TILE_SPIKE Then   ' Spikes
				o.ID = 5
			ElseIf tmpF = TILE_MSPIKE Then  ' Moving spikes
				o.ID = 7; o.F = Rand(0, 3)
			ElseIf tmpF = TILE_GEMLOCK Then ' Gem lock
				o.ID = 6; o.F = 0
			ElseIf tmpF = TILE_KEY Then     ' Key
				o.ID = 8; o.F = 0
			ElseIf tmpF = TILE_LOCKB Or tmpF = TILE_LOCKT Then ' Key Door
				o.ID = 9; o.C[0] = 0; o.C[1] = 0
				If tmpF = TILE_LOCKB Then o.F = 0 Else o.F = 1
			ElseIf tmpF = TILE_FADER Then   ' Fader
				o.ID = 10; o.F = 0
			EndIf
			
			objList.AddLast o

			If tmpF = TILE_SPIKE Or tmpF = TILE_MSPIKE Or tmpF = TILE_GEMLOCK Or tmpF = TILE_LOCKB Or tmpF = TILE_LOCKT Or tmpF = TILE_FADER Then ' Include in collision
				Map[tmpX, tmpY, tmpL]       = New Tile
				Map[tmpX, tmpY, tmpL].F     = tmpF
				Map[tmpX, tmpY, tmpL].X     = tmpX
				Map[tmpX, tmpY, tmpL].Y     = tmpY
				Map[tmpX, tmpY, tmpL].Rot   = tmpR
				Map[tmpX, tmpY, tmpL].Alp   = tmpA
				Map[tmpX, tmpY, tmpL].Solid = tmpS
			EndIf
			
			Case TILE_START ' Jill start
			pX = 128; pY = 132		
			'scrX = tmpX - 3; scrY = tmpY - 6
			
			Default ' Collision tiles
			Map[tmpX, tmpY, tmpL]       = New Tile
			Map[tmpX, tmpY, tmpL].F     = tmpF
			Map[tmpX, tmpY, tmpL].X     = tmpX
			Map[tmpX, tmpY, tmpL].Y     = tmpY
			Map[tmpX, tmpY, tmpL].Rot   = tmpR
			Map[tmpX, tmpY, tmpL].Alp   = tmpA
			Map[tmpX, tmpY, tmpL].Solid = tmpS
		End Select

		' Animated Tiles
		
		' Torch
		If tmpF = TILE_TORCH Then fireList.AddLast objFire.CreateFire(tmpX, tmpY - 1, Rand(0, 2)) ' Create fire
	
	Else ' Edit mode
		Map[tmpX, tmpY, tmpL]       = New Tile
		Map[tmpX, tmpY, tmpL].F     = tmpF
		Map[tmpX, tmpY, tmpL].X     = tmpX
		Map[tmpX, tmpY, tmpL].Y     = tmpY
		Map[tmpX, tmpY, tmpL].Rot   = tmpR
		Map[tmpX, tmpY, tmpL].Alp   = tmpA
		Map[tmpX, tmpY, tmpL].Solid = tmpS
		
	EndIf

	If tmpX > tmpMX Then tmpMX = tmpX ' Find greatest X
	If tmpY > tmpMY Then tmpMY = tmpY ' Find greatest Y
Until Eof(File)

' Reset vars
maxX = tmpMX; maxY = tmpMY; totalY = 0; totalX = 0
scrollX = 0; scrollY = 0

CloseFile file
End Function	

' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
' // SaveMap
' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
Function SaveMap(myfile$)

Local file, a, b

file = WriteFile("maps/" + myfile$ + ".joj")

For b = 0 To 499 
	For a = 0 To 499
		If Map[a, b, 0] <> Null Then           ' layer 0
			WriteShort file, a                 ' map x
			WriteShort file, b                 ' map y
			WriteByte file, Map[a, b, 0].F     ' Frame
			WriteByte file, 0                  ' Layer
			WriteShort file, Map[a, b, 0].Rot  ' Rotation
			WriteFloat file, Map[a, b, 0].Alp  ' Alpha
			WriteByte file, Map[a, b, 0].Solid ' Solid
		EndIf
		
		If Map[a, b, 1] <> Null Then           ' layer 1
			WriteShort file, a                 ' map x
			WriteShort file, b                 ' map y
			WriteByte file, Map[a, b, 1].F     ' Frame
			WriteByte file, 1                  ' Layer
			WriteShort file, Map[a, b, 1].Rot  ' Rotation
			WriteFloat file, Map[a, b, 1].Alp  ' Alpha
			WriteByte file, Map[a, b, 1].Solid ' Solid
		EndIf
	Next
Next

CloseFile file
End Function

' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
' // SaveScreen
' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
Function SaveScreen()
Local tmpFN% = 0, Dir% = ReadDir("snaps/"), file$
Local tmpP%, tmpU%, tmpN%

Repeat 
	file$ = NextFile(Dir)
	If FileType("snaps/" + file$) = 1 Then
		tmpP = Instr(file$, ".", 1)
		tmpU = Instr(file$, "_", 1)
		tmpN = Int(Mid(file$, tmpU + 1, tmpP - tmpU))
		If tmpN > tmpFN Then tmpFN = tmpN ' New filenumber
	EndIf
Until file$ = ""
CloseDir Dir

SetMaskColor 255, 0, 255
Local imgScreen:TPixMap = GrabPixmap(0, 0, 320, 200)
'imgScreen.Save "snaps/snap_" + (tmpFN + 1) + ".png" ' Save PNG

imgScreen = Null; FlushMem
End Function

' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
' // RenderTiles
' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
Function RenderTiles(layer)

Local tmpX = 88, tmpY = 24, tmp = 0, tmp2 = 0, a, b, waterTmp

If scrX <> 0 Then tmpX = 72; tmp = 1
If scrY <> 0 Then tmpY = 8; tmp2 = 1

SetImageHandle imgTiles, 8, 8; SetImageHandle imgWater, 8, 8
SetImageHandle imgLava, 8, 8

Select layer

	Case 0 ' Layer 0
	For b = scrY - tmp2 To scrY + 11
		For a = scrX - tmp To scrX + 15
			If Map[a, b, 0] <> Null Then ' Layer 0
				Select Map[a, b, 0].F
					Case TILE_SPIKE, TILE_GEMLOCK, TILE_MSPIKE, TILE_LAVA, TILE_FADER, TILE_WATERM, TILE_WATERL, TILE_WATERR, TILE_WATERT ' Excluded objects And animated tiles
					If Map[a, b, 0].F = TILE_LAVA Then ' Lava
						SetRotation 0; SetAlpha Map[a, b, 0].Alp
						DrawImage imgLava, Int(tmpX - scrollX), Int(tmpY - scrollY), lavaF
					ElseIf Map[a, b, 0].F >= TILE_WATERM And Map[a, b, 0].F <= TILE_WATERL ' Water
						SetRotation 0; SetAlpha Map[a, b, 0].Alp
						If Map[a, b, 0].F = TILE_WATERT Then 
							waterTmp = 3
						ElseIf Map[a, b, 0].F = TILE_WATERR Then
							waterTmp = 6
						ElseIf Map[a, b, 0].F = TILE_WATERL Then
							waterTmp = 9
						Else
							waterTmp = 0
						EndIf
						DrawImage imgWater, Int(tmpX - scrollX), Int(tmpY - scrollY), waterF + waterTmp
					EndIf
					
					Default				
					SetRotation Map[a, b, 0].Rot; SetAlpha Map[a, b, 0].Alp
					DrawImage imgTiles, Int(tmpX - scrollX), Int(tmpY - scrollY), Map[a, b, 0].F
				End Select
			EndIf
			tmpX = tmpX + 16
		Next
		tmpY = tmpY + 16
		If scrX = 0 Then tmpX = 88 Else tmpX = 72
	Next

	Case 1 ' Layer 1
	For b = scrY - tmp2 To scrY + 11
		For a = scrX - tmp To scrX + 15
			If Map[a, b, 1] <> Null Then ' Layer 1
				Select Map[a, b, 1].F
					Case TILE_SPIKE, TILE_GEMLOCK, TILE_MSPIKE, TILE_LAVA, TILE_FADER, TILE_WATERM, TILE_WATERL, TILE_WATERR, TILE_WATERT  ' Excluded objects And animated tiles
					If Map[a, b, 1].F = TILE_LAVA Then ' Lava
						SetRotation 0; SetAlpha Map[a, b, 1].Alp
						DrawImage imgLava, Int(tmpX - scrollX), Int(tmpY - scrollY), lavaF
					ElseIf Map[a, b, 1].F >= TILE_WATERM And Map[a, b, 1].F <= TILE_WATERL ' Water
						SetRotation 0; SetAlpha Map[a, b, 1].Alp
						If Map[a, b, 1].F = TILE_WATERT Then 
							waterTmp = 3
						ElseIf Map[a, b, 1].F = TILE_WATERR Then
							waterTmp = 6
						ElseIf Map[a, b, 1].F = TILE_WATERL Then
							waterTmp = 9
						Else
							waterTmp = 0
						EndIf
						DrawImage imgWater, Int(tmpX - scrollX), Int(tmpY - scrollY), waterF + waterTmp
					EndIf
					
					Default				
					SetRotation Map[a, b, 1].Rot; SetAlpha Map[a, b, 1].Alp
					DrawImage imgTiles, Int(tmpX - scrollX), Int(tmpY - scrollY), Map[a, b, 1].F
				End Select
			EndIf
			tmpX = tmpX + 16
		Next
		tmpY = tmpY + 16
		If scrX = 0 Then tmpX = 88 Else tmpX = 72
	Next

End Select

SetRotation 0; SetAlpha 1.0; SetImageHandle imgTiles, 0, 0; SetImageHandle imgWater, 0, 0
SetImageHandle imgLava, 0, 0

End Function

' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
' // RenderHealth
' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
Function RenderHealth()

If pLife > 0 Then
	SetColor 244, 0, 44; DrawLine 49, 109, 49 + (pLife / 10), 109
	DrawLine 49 + (pLife / 10), 109, 49 + (pLife / 10), 114
	SetColor 140, 0, 44; DrawLine 49, 114, 48 + (pLife / 10), 114
	SetColor 192, 0, 44; DrawRect 49, 110, (pLife / 10), 4
EndIf

SetColor 255, 255, 255
End Function

' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
' // Legacy Functions
' //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%//
Function RectsOverlap%(x0, y0, w0, h0, x2, y2, w2, h2)
	If x0 > (x2 + w2) Or (x0 + w0) < x2 Then Return False
	If y0 > (y2 + h2) Or (y0 + h0) < y2 Then Return False
	Return True
End Function

Function ImageRectCollide%(image:TImage, ix%, iy%, iframe%, rx%, ry%, rwidth%, rheight%)

   ResetCollisions
   CollideImage image, ix, iy, iframe, 0, 1
   
   If CollideRect(rx, ry, rwidth, rheight, 1, 0) Then Return True Else Return False

End Function 

Function MilliSecs()

	If GetTickCount() > LastMilliSecs Then MyMilliSecs = MyMilliSecs + (GetTickCount() - LastMilliSecs)
	If GetTickCount() < LastMilliSecs Then MyMilliSecs = MyMilliSecs + (LastMilliSecs - GetTickCount())

	LastMilliSecs = GetTickCount()
	Return MyMilliSecs
	
End Function
